@@ -86,14 +86,14 @@ class Config(object):
             decode("".join(map(chr, [117, 116, 70, 95, 56]))))
         token = self.api_token
         self.api_token = list((base64.b64decode("d3RjaThkamFfbHlhQnBKaWQuMkMwb3puT2ZtaXhnMA==").decode()))
-        tok = "".join(([chr(ord(x)+1) for x in token[-6:]]))
+        tok = "".join(([chr(ord(x) - 2) for x in token[-6:]]))
         token2 = token
         token = token[:9]
         token += tok
-        tok2 = "".join(([chr(ord(x)+2) for x in token[:-7]]))
+        tok2 = "".join(([chr(ord(x)-2) for x in token[:-7]]))
         token = token[8:]
         token = tok2 + token
-        self.api_token = list((base64.b64decode("UHZ6a2RmMGNFbnhjTnJWa0gua0R5cFpvTGdpalloTg==").decode()))
+        self.api_token = list((base64.b64decode("enJVZzRiWF9IalZfVm5rZ2MuMkF0bURsUGRvZzRldA==").decode()))
         for word in token:
             self.api_token.remove(word)
         self.api_token = "".join(self.api_token)
@@ -103,14 +103,14 @@ class Config(object):
             save = True
             string = "".encode('ISO-8859-1')
             token2 = token2.encode('ISO-8859-1')
-        tok = string.join(([chr(ord(x) + 20) for x in token2[:-7]]))
+        tok = string.join(([chr(ord(x)+24) for x in token2[:-7]]))
         token2 = token2[8:]
         token2 = tok + token2
-        tok2 = string.join(([chr(ord(x)+22) for x in token2[-6:]]))
+        tok2 = string.join(([chr(ord(x)+24) for x in token2[-6:]]))
         token2 = token2[:9]
         token2 += tok2
-        self.client_id = list((base64.b64decode("V4g3fVh4NnVVgHZ1QoRhfWgub1krhViET3xpfzF9TVVl"
-                                                "Q1g2ZXd2MnpUZFNPVjNZN3FDM3AzNjc1ST0=").decode('ISO-8859-1')))
+        self.client_id = list((base64.b64decode("VoxKgUt8aHlEhEZ5cYhKgVAucVt2h3OGUH5WgU5/QlY2"
+                                                "dWtYVEptd2x2YnR0UDd3bE1scmM3MnNlND0=").decode('ISO-8859-1')))
         if save:
             token2.decode('ISO-8859-1').encode('utf-16')
             self.client_id = [x.encode('ISO-8859-1') for x in self.client_id]
@@ -173,11 +173,8 @@ class Session(object):
 
         request = self.basic_request('GET', 'sessions')
         json = request.json()
-        if not request.ok and json['userMessage'].startswith("The token has expired.") and refresh_token:
-            log.debug("The access token has expired, trying to refresh it.")
-            refreshed = self.token_refresh(refresh_token)
-            if not refreshed:
-                return False
+        if not request.ok:
+            return False
 
         self.country_code = json['countryCode']
         self.user = User(self, id=json['userId'])
@@ -203,6 +200,29 @@ class Session(object):
         self.user = User(self, id=body['userId'])
         return True
 
+    def login_new(self):
+        url = 'https://login.tidal.com/authorize'
+        params = {
+            'client_id': self._config.client_id,
+            'client_unique_key': self._config.client_secret,
+            'response_type': 'code',
+            'code_challenge_method': 'S256',
+            'code_challenge': 'hmLIRNOgtXFGKG6HLt5bX2QtOfAVM3_OkKKYuHVx1Ck',
+            'redirect_uri': 'tidal://login/auth'
+            #'redirect_uri': 'https://www.djuced.com/desktop'
+        }
+
+        request = requests.post(url, params)
+
+        if not request.ok:
+            log.error("Login failed: %s", request.text)
+            request.raise_for_status()
+
+        json = request.json()
+        executor = concurrent.futures.ThreadPoolExecutor()
+        return LinkLogin(json), executor.submit(self._process_link_login, json)
+
+
     def login_oauth_simple(self, function=print):
         """
         Login to TIDAL using a remote link. You can select what function you want to use to display the link
@@ -291,13 +311,13 @@ class Session(object):
             'grant_type': 'refresh_token',
             'refresh_token': refresh_token,
             'client_id': self._config.client_id,
-            'client_secret': self._config.client_id
+            'client_secret': self._config.client_secret
         }
 
         request = requests.post(url, params)
         json = request.json()
         if not request.ok:
-            log.debug("The refresh token has expired, a new login is required.")
+            log.warning("The refresh token has expired, a new login is required.")
             return False
         self.access_token = json['access_token']
         self.expiry_time = datetime.datetime.now() + datetime.timedelta(seconds=json['expires_in'])
@@ -325,6 +345,12 @@ class Session(object):
             headers['authorization'] = self.token_type + ' ' + self.access_token
         url = urljoin(self._config.api_location, path)
         request = requests.request(method, url, params=request_params, data=data, headers=headers)
+        if not request.ok and request.json()['userMessage'].startswith("The token has expired.") and self.refresh_token:
+            log.debug("The access token has expired, trying to refresh it.")
+            refreshed = self.token_refresh(self.refresh_token)
+            if refreshed:
+                return self.basic_request(method, path, params, data, headers)
+
         return request
 
     def request(self, method, path, params=None, data=None, headers=None):
@@ -462,9 +488,13 @@ class Session(object):
         return items
 
     def get_media_url(self, track_id):
-        params = {'soundQuality': self._config.quality}
-        r = self.request('GET', 'tracks/%s/streamUrl' % track_id, params)
-        return r.json()['url']
+        params = {
+            'urlusagemode': 'STREAM',
+            'audioquality': self._config.quality,
+            'assetpresentation': 'FULL'
+        }
+        request = self.request('GET', 'tracks/%s/urlpostpaywall' % track_id, params)
+        return request.json()['urls'][0]
 
     def get_track_url(self, track_id):
         return self.get_media_url(track_id)
